unit nlvcore;

interface

uses
  CFront,
  Classes,
  Contnrs;

type
  TLicenseGranule = class(TObject)
  private
    FID: integer;
    FName: string;
    FAmount: integer;
    FExpires: TDateTime;
  public
    constructor Create(Text: string);
    property ID: integer read FID;
    property Name: string read FName;
    property Amount: integer read FAmount;
    property Expires: TDateTime read FExpires;
  end;

  {$MINENUMSIZE 4}

  TObjectType = (otTableData, otTable, otForm, otReport, otDataport, otCodeunit, otSystem = 10, otFieldNumber);
  TObjectID = 1..MaxInt;
	TPermission = (pmRead, pmInsert, pmModify, pmDelete, pmExecute, pmReadIndirect, pmInsertIndirect, pmModifyIndirect, pmDeleteIndirect, pmExecuteIndirect);
	TPermissions = set of TPermission;

	TPermissionRange = class(TObject)
  private
    FObjectType: TObjectType;
    FFromID: TObjectID;
    FToID: TObjectID;
    FPermissions: TPermissions;
  protected
    function GetPermissionStr: string; virtual;
    function GetObjectTypeStr: string; virtual;
  public
    constructor Create(hTable: TDBL_HTABLE; hRecord: TDBL_HREC);
    property ObjectType: TObjectType read FObjectType;
    property ObjectTypeStr: string read GetObjectTypeStr;
    property FromID: TObjectID read FFromID;
    property ToID: TObjectID read FToID;
    property Permissions: TPermissions read FPermissions;
    property PermissionStr: string read GetPermissionStr;
  end;

  TGranulesSortBy = (gsID, gsName, gsAmount, gsExpires);

  TNavisionLicenseFile = class(TObject)
  private
    FFileName: string;
    FLicenseNo: string;
    FLicensee: TStringList;
    FGranules: TObjectList;
    FPermissionRanges: TObjectList;
    FGranulesSortBy: TGranulesSortBy;
  protected
    function GetGranule(Index: integer): TLicenseGranule; virtual;
    function GetGranuleCount: integer; virtual;
    function GetPermissionRange(Index: integer): TPermissionRange; virtual;
    function GetPermissionRangeCount: integer; virtual;
    procedure ParseLicenseNo(Line: string); virtual;
    procedure ParseLicensee(Line: string; FirstLine: boolean); virtual;
    procedure SetGranulesSortBy(Value: TGranulesSortBy); virtual;
  public
    constructor Create;
    destructor Destroy; override;
    procedure LoadFromFile(const FileName: string); virtual;
    property FileName: string read FFileName;
    property Licensee: TStringList read FLicensee;
    property LicenseNo: string read FLicenseNo;
    property GranuleCount: integer read GetGranuleCount;
    property Granules[Index: integer]: TLicenseGranule read GetGranule;
    property GranulesSortBy: TGranulesSortBy read FGranulesSortBy write SetGranulesSortBy;
		property PermissionRangeCount: integer read GetPermissionRangeCount;
		property PermissionRanges[Index: integer]: TPermissionRange read GetPermissionRange;
  end;

implementation

uses
  Math,
  DateUtils,
  StrUtils,
  SysUtils,
  Dialogs;

{ TNavisionLicenseFile }

constructor TNavisionLicenseFile.Create;
begin
  inherited Create;
  FLicensee := TStringList.Create;
  FGranules := TObjectList.Create;
  FPermissionRanges := TObjectList.Create;
  DBL_Init;
end;

destructor TNavisionLicenseFile.Destroy;
begin
  FLicensee.Free;
  FGranules.Free;
  FPermissionRanges.Free;
  DBL_Exit;
  inherited;
end;

function TNavisionLicenseFile.GetGranule(Index: integer): TLicenseGranule;
begin
  Result := FGranules.Items[Index] as TLicenseGranule;
end;

function TNavisionLicenseFile.GetGranuleCount: integer;
begin
  Result := FGranules.Count;
end;

function TNavisionLicenseFile.GetPermissionRange(Index: integer): TPermissionRange;
begin
  Result := FPermissionRanges.Items[Index] as TPermissionRange;
end;

function TNavisionLicenseFile.GetPermissionRangeCount: integer;
begin
  Result := FPermissionRanges.Count;
end;

procedure TNavisionLicenseFile.LoadFromFile(const FileName: string);
var
  hTable: TDBL_HTABLE;
  hRecord: TDBL_HREC;
  First: string;
  Line: array[0..250] of char;
begin
  FFileName := FileName;
  DBL_LoadLicenseFile(PDBL_U8(FileName));

  FGranules.Clear;
  DBL_OpenTable(@hTable, 2000000040);
  try
    hRecord := DBL_AllocRec(hTable);
    try
      First := '-';
      if DBL_FindRec(hTable, hRecord, PDBL_U8(First)) then
      begin
        DBL_NextRec(hTable, hRecord, 3);

        DBL_GetFieldData(@Line, SizeOf(Line), hTable, hRecord, 2);
        ParseLicenseNo(Line);

        DBL_NextRec(hTable, hRecord, 1);
        DBL_GetFieldData(@Line, SizeOf(Line), hTable, hRecord, 2);
        ParseLicensee(Line, true);

        DBL_NextRec(hTable, hRecord, 1);
        DBL_GetFieldData(@Line, SizeOf(Line), hTable, hRecord, 2);
        while Line[1] <> '*' do
        begin
          ParseLicensee(Line, false);
          DBL_NextRec(hTable, hRecord, 1);
          DBL_GetFieldData(@Line, SizeOf(Line), hTable, hRecord, 2);
        end;

        DBL_NextRec(hTable, hRecord, 3);
        repeat
          DBL_GetFieldData(@Line, SizeOf(Line), hTable, hRecord, 2);
          if Line[1] = '*' then
            Break;
          FGranules.Add(TLicenseGranule.Create(Line));
        until DBL_NextRec(hTable, hRecord, 1) = 0;
			end;
    finally
      DBL_FreeRec(hRecord);
    end;
  finally
    DBL_CloseTable(hTable);
  end;

  FPermissionRanges.Clear;
  DBL_OpenTable(@hTable, 2000000044);
  try
    hRecord := DBL_AllocRec(hTable);
    try
      First := '-';
      if DBL_FindRec(hTable, hRecord, PDBL_U8(First)) then
        repeat
          FPermissionRanges.Add(TPermissionRange.Create(hTable, hRecord));
        until DBL_NextRec(hTable, hRecord, 1) = 0;
    finally
      DBL_FreeRec(hRecord);
    end;
  finally
    DBL_CloseTable(hTable);
  end;
end;

procedure TNavisionLicenseFile.ParseLicensee(Line: string;
  FirstLine: boolean);
var
  NamePos: integer;
begin
  if FirstLine then
    FLicensee.Clear;

  if FirstLine then
  begin
    NamePos := Pos(':', Line);
    if NamePos <> 0 then
      Delete(Line, 1, NamePos);
  end;

  Line := Trim(Line);
  if Line <> '' then
    FLicensee.Add(Line);
end;

procedure TNavisionLicenseFile.ParseLicenseNo(Line: string);
var
	NoPos: integer;
begin
  FLicenseNo := '';

  NoPos := Pos(':', Line);
  if NoPos <> 0 then
  begin
    Delete(Line, 1, NoPos);
    FLicenseNo := Trim(Line);
  end;
end;

function SortGranulesByID(Item1, Item2: Pointer): integer;
var
  Granule1, Granule2: TLicenseGranule;
begin
  Granule1 := TLicenseGranule(Item1);
  Granule2 := TLicenseGranule(Item2);
  Result := CompareValue(Granule1.ID, Granule2.ID);
end;

function SortGranulesByName(Item1, Item2: Pointer): integer;
var
  Granule1, Granule2: TLicenseGranule;
begin
  Granule1 := TLicenseGranule(Item1);
  Granule2 := TLicenseGranule(Item2);
  Result := CompareText(Granule1.Name, Granule2.Name);
end;

function SortGranulesByAmount(Item1, Item2: Pointer): integer;
var
  Granule1, Granule2: TLicenseGranule;
begin
  Granule1 := TLicenseGranule(Item1);
  Granule2 := TLicenseGranule(Item2);
  Result := CompareValue(Granule1.Amount, Granule2.Amount);
end;

function SortGranulesByExpiry(Item1, Item2: Pointer): integer;
var
  Granule1, Granule2: TLicenseGranule;
begin
  Granule1 := TLicenseGranule(Item1);
  Granule2 := TLicenseGranule(Item2);
  Result := CompareDateTime(Granule1.Expires, Granule2.Expires);
end;

procedure TNavisionLicenseFile.SetGranulesSortBy(Value: TGranulesSortBy);
begin
  FGranulesSortBy := Value;

  case GranulesSortBy of
    gsID: FGranules.Sort(SortGranulesByID);
    gsName: FGranules.Sort(SortGranulesByName);
		gsAmount: FGranules.Sort(SortGranulesByAmount);
		gsExpires: FGranules.Sort(SortGranulesByExpiry);
	end;
end;

{ TPermissionRange }

constructor TPermissionRange.Create(hTable: TDBL_HTABLE;
	hRecord: TDBL_HREC);
var
	Read, Insert, Modify, Delete, Execute: integer;
begin
	inherited Create;
	DBL_GetFieldData(@FObjectType, SizeOf(FObjectType), hTable, hRecord, 1);
	DBL_GetFieldData(@FFromID, SizeOf(FFromID), hTable, hRecord, 3);
	DBL_GetFieldData(@FToID, SizeOf(FToID), hTable, hRecord, 4);
	DBL_GetFieldData(@Read, SizeOf(Read), hTable, hRecord, 10);
	DBL_GetFieldData(@Insert, SizeOf(Insert), hTable, hRecord, 11);
	DBL_GetFieldData(@Modify, SizeOf(Modify), hTable, hRecord, 12);
	DBL_GetFieldData(@Delete, SizeOf(Delete), hTable, hRecord, 13);
	DBL_GetFieldData(@Execute, SizeOf(Execute), hTable, hRecord, 14);
	case Read of
		1: FPermissions := FPermissions + [pmRead];
		2: FPermissions := FPermissions + [pmReadIndirect];
	end;
	case Insert of
		1: FPermissions := FPermissions + [pmInsert];
		2: FPermissions := FPermissions + [pmInsertIndirect];
	end;
	case Modify of
		1: FPermissions := FPermissions + [pmModify];
		2: FPermissions := FPermissions + [pmModifyIndirect];
	end;
	case Delete of
		1: FPermissions := FPermissions + [pmDelete];
		2: FPermissions := FPermissions + [pmDeleteIndirect];
	end;
	case Execute of
		1: FPermissions := FPermissions + [pmExecute];
		2: FPermissions := FPermissions + [pmExecuteIndirect];
	end;
end;

function TPermissionRange.GetObjectTypeStr: string;
resourcestring
	sTableData = 'TableData';
	sTable = 'Table';
	sForm = 'Form';
	sReport = 'Report';
	sDataport = 'Dataport';
	sCodeunit = 'Codeunit';
	sSystem = 'System';
	sFieldNumber = 'FieldNumber';
const
	ObjTypeStr: array[TObjectType] of string = (sTableData, sTable, sForm, sReport, sDataport, sCodeunit, '', '', '', '', sSystem, sFieldNumber);
begin
	Result := ObjTypeStr[ObjectType];
end;

function TPermissionRange.GetPermissionStr: string;
begin
	if pmRead in Permissions then Result := Result + 'R';
	if pmReadIndirect in Permissions then Result := Result + 'r';
	if pmInsert in Permissions then Result := Result + 'I';
	if pmInsertIndirect in Permissions then Result := Result + 'i';
	if pmModify in Permissions then Result := Result + 'M';
	if pmModifyIndirect in Permissions then Result := Result + 'm';
	if pmDelete in Permissions then Result := Result + 'D';
	if pmDeleteIndirect in Permissions then Result := Result + 'd';
	if pmExecute in Permissions then Result := Result + 'X';
	if pmExecuteIndirect in Permissions then Result := Result + 'x';
end;

{ TLicenseGranule }

constructor TLicenseGranule.Create(Text: string);
var
  i: integer;
  DateStr: string;
  AmountStr: string;
function HasAmount: boolean;
begin
  Result := (Copy(Text, 67, 1) <> '');
end;
begin
  inherited Create;
  FID := StrToInt(Trim(StringReplace(Copy(Text, 1, 14), ',', '', [rfReplaceAll])));

  case HasAmount of
    true:
    begin
      i := 62;
      while (PosEx(' ', Text, i) <> 0) and (i <= 66) do
        Inc(i);
      FName := Trim(Copy(Text, 16, i - 16));
      AmountStr := Copy(Text, i, 68 - i);
    end;
    false:
    begin
      FName := Trim(Copy(Text, 16, 52));
      AmountStr := '';
    end;
  end;

  AmountStr := Trim(StringReplace(AmountStr, ',', '', [rfReplaceAll]));
  if (AmountStr <> '') then
    FAmount := StrToInt(AmountStr);

  DateStr := Trim(Copy(Text, 69, 8));
  DateStr := StringReplace(DateStr, '.', DateSeparator, [rfReplaceAll]);
  DateStr := StringReplace(DateStr, '/', DateSeparator, [rfReplaceAll]);
  DateStr := StringReplace(DateStr, '-', DateSeparator, [rfReplaceAll]);
  if DateStr <> '' then
    FExpires := StrToDate(DateStr);
end;

end.
